/**
 * File: gps.cpp
 * Version: 0.1
 * Authors: Stéphane Cousot <http://www.ubaa.net/>
 * License: gps is released under GNU General Public License, version 3 or later
 *
 * Read the latitude and longitude from the GPS device and save both on the remote server.
 *Usage: [-v|-c] [-f seconds]
 *	-v outputs the serial stream to the console
 *	-c outputs the result to the console and disables online recording
 *	-f forces the update frequency to the specify value (in seconds).
 *	   set to 0 to retrieve data once.
 * 
 * About the output message in the console mode.
 *	- first line: all data separated by commas
 *		- 1 report the GPS status: A-ok, V-invalid, D-no device found or unable to open the GPS device
 *		- 2 current Latitude
 *		- 3 current Longitude
 *	- second line: the original GPS data or the error message on failure
 *
 * -------------------------------------------------------------------------------------------------------
 *
 * 	- to install dependencies:
 * 	  pacman -S wiringpi gcc
 * 	- to compile this script:
 * 	  c++ -std=gnu++11 -o /home/locusonus/bin/gps /home/locusonus/dev/gps.cpp -lcurl -lwiringPi -lsystemd 
 *
 * --------------------------------------------------------------------------------------------------------
 *
 *	This file is part of the Locus Sonus streambox project.
 *	for more informations, see <http://locusonus.org/streambox/>
 *
 *	This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#define TIMEOUT 	 40
#define DEFAULT_FREQ 60


#include <wiringSerial.h>
#include <curl/curl.h>
#include <systemd/sd-journal.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <signal.h>
#include <fstream>
#include <regex>



using namespace std;

// globals options
bool verbose    = false;	// verbose mode
bool dosave     = true;		// whether the result should be saved to the remote database or output to the console
bool is_service = false;	// whether this program is running insde systemd service or not

volatile sig_atomic_t run = 0;
static void sighandler( int signum ) { run=0; }


int getStreamID()
{
	smatch match;
	string line;
	string err;
	string sid;

	ifstream conf("/boot/darkice.cfg");
	regex pattern("^streamID.*=\\s+(.*)");

	if ( conf.is_open() ) {
		while( !conf.eof() ) {
			getline( conf, line );
			regex_search( line, match, pattern );
			if ( match.size() ) sid = match[1];
		}
		if ( sid.empty() ) err = "bad configuration";
	}
	else err = "configuration file not found";

	// print error
	if ( !err.empty() ) {
		if (!is_service) fprintf(stderr, "Failed to get the Streambox identifier: %s\n", err.c_str());
		else sd_journal_print(LOG_ERR, "Failed to get the Streambox identifier: %s\n", err.c_str());
		return -1;
	}

	conf.close();
	return stoi(sid);
}

int getUpdateFrequency()
{
	smatch match;
	string line;
	string err;
	string freq;

	ifstream conf("/boot/streambox.txt");
	regex pattern("^GPS_FREQUENCY=\"(.*)\"");

	if ( conf.is_open() ) {
		while( !conf.eof() ) {
			getline( conf, line );
			regex_search( line, match, pattern );
			if ( match.size() ) {
				freq = match[1];
				if ( freq.empty() ) return 0;
			}
		}
		if ( freq.empty() ) err = "incorrect value";
	}
	else err = "streambox settings not found";

	// print error
	if ( !err.empty() ) {
		if (dosave) {
			if (!is_service) fprintf(stderr, "Failed to get GPS frequency setting: %s\n", err.c_str());
			else sd_journal_print(LOG_ERR, "Failed to get GPS frequency setting: %s\n", err.c_str());
		}
		return DEFAULT_FREQ;
	}

	conf.close();
	return stoi(freq);
}

int saveGPSLocation(int id, float lat, float lon)
{
	CURL *curl;
	CURLcode res;
	
	long code = 0;
	char url[1024];

	curl = curl_easy_init();
	if ( curl ) {
		sprintf( url, "https://locusonus.org/soundmap/admin/location/%i/%f/%f", id, lat, lon);
		curl_easy_setopt( curl, CURLOPT_URL, url );
		curl_easy_setopt( curl, CURLOPT_CONNECTTIMEOUT, TIMEOUT );
		curl_easy_setopt( curl, CURLOPT_NOBODY, 1 );
		res = curl_easy_perform(curl);
		curl_easy_getinfo( curl, CURLINFO_RESPONSE_CODE, &code );
		curl_easy_cleanup(curl);

		if (code==200) {
			if (!is_service) fprintf(stdout, "Geolocation saved @ %f,%f\n", lat, lon);
			else sd_journal_print(LOG_INFO,  "Geolocation saved @ %f,%f\n", lat, lon);
		}
		else {
			if (!is_service) fprintf(stderr, "Failed to save geolocation data, error code: %lu", code );
			else sd_journal_print(LOG_ERR, "Failed to save geolocation data, error code: %lu", code );
		}
		
		return code;
	}

	return 0;
}

float convertToDD(string degrees, string minutes, string direction)
{
	float out = stoi(degrees) + (stof(minutes) / 60);
	if (direction == "S" || direction == "W") out = out * -1.0;
	return out;
}

// http://aprs.gids.nl/nmea/#rmc
// https://docs.novatel.com/OEM7/Content/Logs/GPRMC.htm
int parseGPSData(const char* GPRMC, float* latlon)
{
	vector<string> fields;

	string str;
	stringstream ss(GPRMC);

	// split data comma delimiter
	while (getline(ss, str, ',')) fields.push_back(str);

	// GPRMC status (A = data valid, V = data invalid)
	// convert DDDmm.mm coordinates to DD (decimal degrees) and DMS (degrees, minutes, seconds)
	bool valid = fields[2][0] == 'A';
	if (valid) {
		string latdir = fields[4];
		string londir = fields[6];
		string latdeg = fields[3].substr(0, 2);
		string latmin = fields[3].substr(2, 10);
		string londeg = fields[5].substr(0, 3);
		string lonmin = fields[5].substr(3, 11);

		latlon[0] = convertToDD(latdeg, latmin, latdir);
		latlon[1] = convertToDD(londeg, lonmin, londir);
	}

	// display result
	if (verbose) {
		printf("\n***********************\n\n");
		printf("status: %s = %s\n", fields[2].c_str(), valid ? "valid" : "invalid");
		if (valid) {
			printf("latitude: %f\n", latlon[0]);
			printf("longitude: %f\n", latlon[1]);
		}
		printf("\n***********************\n\n");
	}

	return valid;
}


int main(int argc, char *argv[])
{
	int freq = -1;

	int opt;
	while( (opt=getopt(argc, argv, "vcf:h"))!=-1 ) switch(opt)
	{
		case 'v':
			verbose = true;
			break;
		case 'c':
			dosave = false;
			break;
		case 'f':
			freq = stoi(optarg);
			break;
		case 'h':
		case '?':
			printf( "usage: [-v|-c] [-f seconds]\n" );
			printf( "-v outputs the serial stream to the console\n" );
			printf( "-c outputs the result to the console and disables online recording\n" );
			printf( "-f forces the update frequency to the specify value (in seconds)\n" );
			return 0;
	}


	// running as service or in console
	char* val = getenv("SYSTEMD_EXEC_PID");
	is_service = val != NULL;


	// retrieve user ID
	int id = getStreamID();
	if (id==-1) return 1;

	// retrieve refresh frequency
	if (freq==-1) freq = getUpdateFrequency();
	if (freq<0) freq = DEFAULT_FREQ;
	if (dosave) {
		if (freq) {
			if (!is_service) fprintf(stdout, "Set the update frequency to %i seconds\n", freq);
			else sd_journal_print(LOG_INFO,  "Set the update frequency to %i seconds", freq);
		}
		else {
			if (!is_service) fprintf(stdout, "Update Streambox location once\n", freq);
			else sd_journal_print(LOG_INFO,  "Update Streambox location once", freq);
		}
	}

	// opens and initialises the serial device and sets the baud rate
	int fd = serialOpen("/dev/ttyACM0", 9600);
	if (fd<0) {
		const char* msg = strerror(errno);
		if (dosave) {
			if (!is_service) fprintf(stderr, "Unable to open serial device: %s\n", msg);
			else sd_journal_print(LOG_ERR, "Unable to open serial device: %s\n", msg);
		}
		else fprintf(stderr, "D,,\n%s\n", msg);
		return 1;
	}


	char data[255];		// allocate some space for the string
	char c = -1;		// the character read from serial stream
	int ptr = 0;		// index into array where to store the character read

	run = 1;
	while(run) {
		while (serialDataAvail(fd)) {
			c = serialGetchar(fd);
			data[ptr++] = c;
			data[ptr] = '\0';

			if (c == 13) {
				if (verbose) fprintf(stdout, "%s\n", data);
				if ( strncmp(data,"$GPRMC",6)==0 ) {
					float* latlon = new float[2];
					int res = parseGPSData(data, latlon);
					if (res) {
						if (dosave) saveGPSLocation(id, latlon[0], latlon[1]);
						else fprintf(stdout, "A,%f,%f\n%s\n", latlon[0], latlon[1], data);
					}
					else if (!dosave) fprintf(stdout, "V,,\n%s\n", data);

					// wait or quite
					if (freq) sleep(freq);
					else {
						run = 0;
						break;
					}
				}
			}
			if (c == 13 || c == 10) ptr = 0;
		}
	}

	// closes the device
	if (dosave) {
		if (!is_service) fprintf(stdout, "%s\n", "Close device");
		else sd_journal_print(LOG_INFO,  "%s\n", "Close device");
	}
	serialClose(fd);
	
	return 0;
}
